@@ -15,36 +15,38 @@ class SoundSourceProxy;
 class AnalyserQueue : public QThread {
     Q_OBJECT
 
-public:
+    public:
 	AnalyserQueue();
 	virtual ~AnalyserQueue();
-    
-	void queueAnalyseTrack(TrackInfoObject* tio);
 	void stop();
-    
+    int numQueuedTracks();
+
 	static AnalyserQueue* createDefaultAnalyserQueue(ConfigObject<ConfigValue> *_config);
-	static AnalyserQueue* createBPMAnalyserQueue(ConfigObject<ConfigValue> *_config);
+	static AnalyserQueue* createPrepareViewAnalyserQueue(ConfigObject<ConfigValue> *_config);
     static AnalyserQueue* createAnalyserQueue(QList<Analyser*> analysers);
-    
+
+public slots:
+    void queueAnalyseTrack(TrackPointer tio);
+
 signals:
-    void trackProgress(TrackInfoObject*,int);
-    void trackFinished(TrackInfoObject*);
-    
+    void trackProgress(TrackPointer pTrack,int progress);
+    void trackFinished(TrackPointer pTrack);
+
 protected:
 	void run();
-    
+
 private:
 	void addAnalyser(Analyser* an);
-    
+
 	QList<Analyser*> m_aq;
-    
-	TrackInfoObject* dequeueNextBlocking();
-	void doAnalysis(TrackInfoObject* tio, SoundSourceProxy *pSoundSource);
-    
+
+	TrackPointer dequeueNextBlocking();
+	void doAnalysis(TrackPointer tio, SoundSourceProxy *pSoundSource);
+
 	bool m_exit;
-    
+
 	// The processing queue and associated mutex
-	QQueue<TrackInfoObject*> m_tioq;
+	QQueue<TrackPointer> m_tioq;
 	QMutex m_qm;
 	QWaitCondition m_qwait;
 };
