@@ -25,35 +25,34 @@
 #include "widget/woverview.h"
 #include "mixxx.h"
 #include "controlnull.h"
-#include "readerextractwave.h"
 #include "controlpotmeter.h"
 #include "controlobjectthreadmain.h"
-#include "reader.h"
 #include "engine/enginebuffer.h"
 #include "analyserqueue.h"
 #include "engine/enginevumeter.h"
-#include "track.h"
-#include "trackcollection.h"
 #include "trackinfoobject.h"
-#include "bpm/bpmdetector.h"
 #include "dlgabout.h"
 #include "waveform/waveformrenderer.h"
+#include "soundsourceproxy.h"
+
+#include "player.h"
+#include "library/library.h"
+#include "library/librarytablemodel.h"
+#include "library/libraryscanner.h"
+#include "library/legacylibraryimporter.h"
 
 #include "soundmanager.h"
 #include "defs_urls.h"
-#include "defs_audiofiles.h"
-#include "mididevicehandler.h"
 #include "recording/defs_recording.h"
 
+#include "midi/mididevicemanager.h"
 #include "defs_version.h"
 #include "upgrade.h"
 
 #include "build.h" //#defines of details of the build set up (flags, repo number, etc). This isn't a real file, SConscript generates it and it probably gets placed in $PLATFORM_build/. By including this file here and only here we make sure that updating src or changing the build flags doesn't force a rebuild of everything
 
 #ifdef __IPOD__
-#include "wtracktableview.h"
 #include "gpod/itdb.h"
-#include "track.h"
 #endif
 
 #ifdef __C_METRICS__
@@ -71,35 +70,47 @@ extern "C" void crashDlg()
 MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
 {
     app = a;
-    
+
     QString buildRevision, buildFlags;
     #ifdef BUILD_REV
       buildRevision = BUILD_REV;
     #endif
-    
+
     #ifdef BUILD_FLAGS
       buildFlags = BUILD_FLAGS;
     #endif
-    
+
     if (buildRevision.trimmed().length() > 0) {
         if (buildFlags.trimmed().length() > 0)
             buildRevision = "(bzr r" + buildRevision + "; built on: " + __DATE__ + " @ " + __TIME__ + "; flags: " + buildFlags.trimmed() + ") ";
         else
             buildRevision = "(bzr r" + buildRevision + "; built on: " + __DATE__ + " @ " + __TIME__ + ") ";
     }
-    
+
     qDebug() << "Mixxx" << VERSION << buildRevision << "is starting...";
+    qDebug() << "Qt version is:" << qVersion();
+
+    QCoreApplication::setApplicationName("Mixxx");
+    QCoreApplication::setApplicationVersion(VERSION);
+#if defined(AMD64) || defined(EM64T) || defined(x86_64)
+    setWindowTitle(tr("Mixxx " VERSION " x64"));
+#elif defined(IA64)
+    setWindowTitle(tr("Mixxx " VERSION " Itanium"));
+#else
     setWindowTitle(tr("Mixxx " VERSION));
-    setWindowIcon(QIcon(":/images/icon.svg"));
+#endif
+    setWindowIcon(QIcon(":/images/ic_mixxx_window.png"));
 
     //Reset pointer to players
     soundmanager = 0;
-    m_pTrack = 0;
     prefDlg = 0;
-    midi = 0;
-    
+    m_pMidiDeviceManager = 0;
+
     // Check to see if this is the first time this version of Mixxx is run after an upgrade and make any needed changes.
-    config = versionUpgrade();  // This static function is located in upgrade.cpp
+    Upgrade upgrader;
+    config = upgrader.versionUpgrade();
+    bool bFirstRun = upgrader.isFirstRun();
+    bool bUpgraded = upgrader.isUpgraded();
     QString qConfigPath = config->getConfigPath();
 
 #ifdef __C_METRICS__
@@ -111,15 +122,15 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
       int dlg = -1;
       while (dlg != 0 && dlg != 1) {
          dlg = QMessageBox::question(this, "Mixxx", "Mixxx's development is driven by community feedback.  At your discretion, Mixxx can automatically send data on your user experience back to the developers. Would you like to help us make Mixxx better by enabling this feature?", "Yes", "No", "Privacy Policy", 0, -1);
-  	 switch (dlg) {
+       switch (dlg) {
            case 0: metricsAgree = "yes";
-  	   case 1: break;
+         case 1: break;
            default: //show privacy policy
-	            QMessageBox::information(this, "Mixxx: Privacy Policy", "Mixxx's development is driven by community feedback.  In order to help improve future versions Mixxx will with your permission collect information on your hardware and usage of Mixxx.  This information will primarily be used to fix bugs, improve features, and determine the system requirements of later versions.  Additionally this information may be used in aggregate for statistical purposes.\n\nThe hardware information will include:\n\t- CPU model and features\n\t- Total/Available Amount of RAM\n\t- Available disk space\n\t- OS version\n\nYour usage information will include:\n\t- Settings/Preferences\n\t- Internal errors\n\t- Internal debugging messages\n\t- Performance statistics (average latency, CPU usage)\n\nThis information will not be used to personally identify you, contact you, advertise to you, or otherwise bother you in any way.\n");
+                QMessageBox::information(this, "Mixxx: Privacy Policy", "Mixxx's development is driven by community feedback.  In order to help improve future versions Mixxx will with your permission collect information on your hardware and usage of Mixxx.  This information will primarily be used to fix bugs, improve features, and determine the system requirements of later versions.  Additionally this information may be used in aggregate for statistical purposes.\n\nThe hardware information will include:\n\t- CPU model and features\n\t- Total/Available Amount of RAM\n\t- Available disk space\n\t- OS version\n\nYour usage information will include:\n\t- Settings/Preferences\n\t- Internal errors\n\t- Internal debugging messages\n\t- Performance statistics (average latency, CPU usage)\n\nThis information will not be used to personally identify you, contact you, advertise to you, or otherwise bother you in any way.\n");
                     break;
-  	 }
+       }
       }
-    }	
+    }
     config->set(ConfigKey("[User Experience]","AgreedToUserExperienceProgram"), ConfigValue(metricsAgree));
 
     // If the user agrees...
@@ -143,7 +154,15 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
     control = new ControlNull();
 
     // Read keyboard configuration and set kdbConfig object in WWidget
-    kbdconfig = new ConfigObject<ConfigValueKbd>(QString(qConfigPath).append("keyboard/").append("Standard.kbd.cfg"));
+    // Check first in user's Mixxx directory
+    QString userKeyboard = QDir::homePath().append("/").append(SETTINGS_PATH).append("Custom.kbd.cfg");
+    if (QFile::exists(userKeyboard)) {
+        qDebug() << "Found and will use custom keyboard preset" << userKeyboard;
+        kbdconfig = new ConfigObject<ConfigValueKbd>(userKeyboard);
+    }
+    else
+        // Otherwise use the default
+        kbdconfig = new ConfigObject<ConfigValueKbd>(QString(qConfigPath).append("keyboard/").append("Standard.kbd.cfg"));
     WWidget::setKeyboardConfig(kbdconfig);
 
     // Sample rate used by Player object
@@ -170,8 +189,6 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
     // Starting the master (mixing of the channels and effects):
     master = new EngineMaster(config, buffer1, buffer2, channel1, channel2, "[Master]");
 
-    // Initialize player device
-
     soundmanager = new SoundManager(config, master);
     soundmanager->queryDevices();
 
@@ -199,10 +216,38 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
     // Use frame as container for view, needed for fullscreen display
     frame = new QFrame;
     setCentralWidget(frame);
-    //move(10,10);
-    // Call inits to invoke all other construction parts
 
-    view=new MixxxView(frame, kbdconfig, qSkinPath, config);
+    m_pLibrary = new Library(this, config, bFirstRun || bUpgraded);
+    qRegisterMetaType<TrackPointer>("TrackPointer");
+
+    //Create the "players" (virtual playback decks)
+    m_pPlayer1 = new Player(config, buffer1, "[Channel1]");
+    m_pPlayer2 = new Player(config, buffer2, "[Channel2]");
+
+    //Connect the player to the track collection so that when a track is unloaded,
+    //it's data (eg. waveform summary) is saved back to the database.
+    connect(m_pPlayer1, SIGNAL(unloadingTrack(TrackPointer)),
+            &(m_pLibrary->getTrackCollection()->getTrackDAO()),
+            SLOT(saveTrack(TrackPointer)));
+    connect(m_pPlayer2, SIGNAL(unloadingTrack(TrackPointer)),
+            &(m_pLibrary->getTrackCollection()->getTrackDAO()),
+            SLOT(saveTrack(TrackPointer)));
+
+    view=new MixxxView(frame, kbdconfig, qSkinPath, config, m_pPlayer1, m_pPlayer2,
+                       m_pLibrary);
+
+    //Scan the library directory.
+    m_pLibraryScanner = new LibraryScanner(m_pLibrary->getTrackCollection());
+
+    //Refresh the library models when the library (re)scan is finished.
+    connect(m_pLibraryScanner, SIGNAL(scanFinished()),
+            m_pLibrary, SLOT(slotRefreshLibraryModels()));
+
+    //Scan the library for new files and directories.
+    m_pLibraryScanner->scan(config->getValueString(ConfigKey("[Playlist]","Directory")));
+
+
+    // Call inits to invoke all other construction parts
 
     // TODO rryan : Move this to WaveformViewerFactory or something.
     /*
@@ -241,49 +286,61 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
         config->set(ConfigKey("[BPM]","AnalyzeEntireSong"),ConfigValue(1));
     }
 
+    // Setup the analyser queue to automatically process new tracks loaded by either player
+    m_pAnalyserQueue = AnalyserQueue::createDefaultAnalyserQueue(config);
+    connect(m_pPlayer1, SIGNAL(newTrackLoaded(TrackPointer)),
+            m_pAnalyserQueue, SLOT(queueAnalyseTrack(TrackPointer)));
+    connect(m_pPlayer2, SIGNAL(newTrackLoaded(TrackPointer)),
+            m_pAnalyserQueue, SLOT(queueAnalyseTrack(TrackPointer)));
+
+
+
     // Initialize track object:
-    m_pTrack = new Track(config->getValueString(ConfigKey("[Playlist]","Listfile")),
-                         view,
-                         config,
-                         buffer1,
-                         buffer2,
-                         AnalyserQueue::createDefaultAnalyserQueue(config));
+    // m_pTrack = new Track(config->getValueString(ConfigKey("[Playlist]","Listfile")),
+    //                      view,
+    //                      config,
+    //                      buffer1,
+    //                      buffer2,
+    //                      AnalyserQueue::createDefaultAnalyserQueue(config));
 
     //WTreeItem::setTrack(m_pTrack);
     // Set up drag and drop to player visuals
+
     if (view->m_pVisualCh1)
-        connect(view->m_pVisualCh1, SIGNAL(trackDropped(QString)), m_pTrack, SLOT(slotLoadPlayer1(QString)));
+        connect(view->m_pVisualCh1, SIGNAL(trackDropped(QString)),
+                this, SLOT(slotLoadPlayer1(QString)));
     if (view->m_pVisualCh2)
-        connect(view->m_pVisualCh2, SIGNAL(trackDropped(QString)), m_pTrack, SLOT(slotLoadPlayer2(QString)));
+        connect(view->m_pVisualCh2, SIGNAL(trackDropped(QString)),
+                this, SLOT(slotLoadPlayer2(QString)));
 
-    // Ensure that visual receive updates when new tracks are loaded
     if (view->m_pWaveformRendererCh1)
-        connect(m_pTrack, SIGNAL(newTrackPlayer1(TrackInfoObject *)), view->m_pWaveformRendererCh1, SLOT(slotNewTrack(TrackInfoObject *)));
+        connect(m_pPlayer1, SIGNAL(newTrackLoaded(TrackPointer)),
+                view->m_pWaveformRendererCh1, SLOT(slotNewTrack(TrackPointer)));
     if (view->m_pWaveformRendererCh2)
-        connect(m_pTrack, SIGNAL(newTrackPlayer2(TrackInfoObject *)), view->m_pWaveformRendererCh2, SLOT(slotNewTrack(TrackInfoObject *)));
-
+          connect(m_pPlayer2, SIGNAL(newTrackLoaded(TrackPointer)),
+                  view->m_pWaveformRendererCh2, SLOT(slotNewTrack(TrackPointer)));
 
+    connect(m_pLibrary, SIGNAL(loadTrackToPlayer(TrackPointer, int)),
+            this, SLOT(slotLoadTrackToPlayer(TrackPointer, int)));
+    connect(m_pLibrary, SIGNAL(loadTrack(TrackPointer)),
+             this, SLOT(slotLoadTrackIntoNextAvailablePlayer(TrackPointer)));
 
     // Setup state of End of track controls from config database
     ControlObject::getControl(ConfigKey("[Channel1]","TrackEndMode"))->queueFromThread(config->getValueString(ConfigKey("[Controls]","TrackEndModeCh1")).toDouble());
     ControlObject::getControl(ConfigKey("[Channel2]","TrackEndMode"))->queueFromThread(config->getValueString(ConfigKey("[Controls]","TrackEndModeCh2")).toDouble());
 
-    // Initialise MIDI and open the MIDI device
-    MidiDeviceHandler * midiHandler = new MidiDeviceHandler();
-    midi = midiHandler->getMidiPtr();
-    QString midiDevice = config->getValueString(ConfigKey("[Midi]","Device"));
-    if (midiDevice != "")
-    	midi->devOpen(midiDevice);
+    // Initialise midi
+    m_pMidiDeviceManager = new MidiDeviceManager(config);
+    //TODO: Try to open MIDI devices?
+    m_pMidiDeviceManager->queryDevices();
+    m_pMidiDeviceManager->setupDevices();
+
 
     // Initialize preference dialog
-    prefDlg = new DlgPreferences(this, view, soundmanager, m_pTrack, midi, config);
+    prefDlg = new DlgPreferences(this, view, soundmanager,
+                                 m_pMidiDeviceManager, config);
     prefDlg->setHidden(true);
 
-#ifdef __LADSPA__
-    ladspaDlg = new DlgLADSPA(this);
-    ladspaDlg->setHidden(true);
-#endif
-
     // Try open player device If that fails, the preference panel is opened.
     while (soundmanager->setupDevices() != 0)
     {
@@ -302,22 +359,25 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
     //grabKeyboard();
 
     // Load tracks in args.qlMusicFiles (command line arguments) into player 1 and 2:
-#ifndef QT3_SUPPORT
+    if (args.qlMusicFiles.count()>0)
+        this->slotLoadPlayer1((args.qlMusicFiles.at(0)));
     if (args.qlMusicFiles.count()>1)
-        m_pTrack->slotLoadPlayer1((*args.qlMusicFiles.at(1)));
-    if (args.qlMusicFiles.count()>2)
-        m_pTrack->slotLoadPlayer2((*args.qlMusicFiles.at(2)));
-#else
-    if (args.qlMusicFiles.count()>1)
-        m_pTrack->slotLoadPlayer1(args.qlMusicFiles.at(1));
-    if (args.qlMusicFiles.count()>2)
-        m_pTrack->slotLoadPlayer2(args.qlMusicFiles.at(2));
-#endif
+        this->slotLoadPlayer2((args.qlMusicFiles.at(1)));
 
     // Initialize visualization of temporal effects
     channel1->setEngineBuffer(buffer1);
     channel2->setEngineBuffer(buffer2);
-    
+
+    //Automatically load specially marked promotional tracks on first run
+    if (bFirstRun || bUpgraded)
+    {
+        QList<TrackPointer> tracksToAutoLoad = m_pLibrary->getTracksToAutoLoad();
+        if (tracksToAutoLoad.count() > 0)
+            m_pPlayer1->slotLoadTrack(tracksToAutoLoad.at(0));
+        if (tracksToAutoLoad.count() > 1)
+            m_pPlayer2->slotLoadTrack(tracksToAutoLoad.at(1));
+    }
+
 #ifdef __SCRIPT__
     scriptEng = new ScriptEngine(this, m_pTrack);
 #endif
@@ -336,8 +396,16 @@ MixxxApp::MixxxApp(QApplication * a, struct CmdlineArgs args)
     if (args.bStartInFullscreen)
         slotOptionsFullScreen(true);
 #ifdef __C_METRICS__
-	cm_writemsg_ascii(MIXXXCMETRICS_MIXXX_CONSTRUCTOR_COMPLETE, "Mixxx constructor complete.");
+    cm_writemsg_ascii(MIXXXCMETRICS_MIXXX_CONSTRUCTOR_COMPLETE, "Mixxx constructor complete.");
 #endif
+
+    // Refresh the GUI (workaround for Qt 4.6 display bug)
+    QString QtVersion = qVersion();
+    if (QtVersion>="4.6.0") {
+        qDebug() << "Qt v4.6.0 or higher detected. Using rebootMixxxView() workaround."
+                 << "\n    (See bug https://bugs.launchpad.net/mixxx/+bug/521509)";
+        rebootMixxxView();
+    }
 }
 
 MixxxApp::~MixxxApp()
@@ -363,10 +431,6 @@ MixxxApp::~MixxxApp()
     qDebug() << "save config, " << qTime.elapsed();
     config->Save();
 
-    qDebug() << "Write track xml, " << qTime.elapsed();
-    m_pTrack->writeXML(config->getValueString(ConfigKey("[Playlist]","Listfile")));
-	m_pTrack->appShuttingDown();
-
     qDebug() << "close soundmanager" << qTime.elapsed();
     soundmanager->closeDevices();
     qDebug() << "soundmanager->close() done";
@@ -375,6 +439,9 @@ MixxxApp::~MixxxApp()
     config->set(ConfigKey("[Controls]","TrackEndModeCh1"), ConfigValue((int)ControlObject::getControl(ConfigKey("[Channel1]","TrackEndMode"))->get()));
     config->set(ConfigKey("[Controls]","TrackEndModeCh2"), ConfigValue((int)ControlObject::getControl(ConfigKey("[Channel2]","TrackEndMode"))->get()));
 
+    qDebug() << "delete MidiDeviceManager";
+    delete m_pMidiDeviceManager;
+
     qDebug() << "delete soundmanager, " << qTime.elapsed();
     delete soundmanager;
     qDebug() << "delete master, " << qTime.elapsed();
@@ -384,6 +451,10 @@ MixxxApp::~MixxxApp()
     qDebug() << "delete channel2, " << qTime.elapsed();
     delete channel2;
 
+    //delete m_pPlayer1;
+    //delete m_pPlayer2;
+    delete m_pAnalyserQueue;
+
     qDebug() << "delete buffer1, " << qTime.elapsed();
     delete buffer1;
     qDebug() << "delete buffer2, " << qTime.elapsed();
@@ -391,16 +462,21 @@ MixxxApp::~MixxxApp()
 
 //    qDebug() << "delete prefDlg";
 //    delete m_pControlEngine;
-//     qDebug() << "delete midi";
-    qDebug() << "delete midi, " << qTime.elapsed();
-    delete midi;
-//    qDebug() << "delete midiconfig";
 
     qDebug() << "delete view, " << qTime.elapsed();
     delete view;
 
-    qDebug() << "delete tracks, " << qTime.elapsed();
-    delete m_pTrack;
+    qDebug() << "delete library scanner" <<  qTime.elapsed();
+    delete m_pLibraryScanner;
+
+    //Delete the library after the view so there are no dangling pointers to the data models.
+    qDebug() << "delete library" << qTime.elapsed();
+    delete m_pLibrary;
+
+    //HACK: Save config again. We saved it once before doing some dangerous stuff. We only really want to
+    //      save it here, but the first one was just a precaution. The earlier one can be removed when
+    //      stuff is more stable at exit.
+    config->Save();
 
     delete prefDlg;
 
@@ -409,24 +485,19 @@ MixxxApp::~MixxxApp()
     delete frame;
 
 #ifdef __C_METRICS__ // cmetrics will cause this whole method to segfault on Linux/i386 if it is called after config is deleted. Obviously, it depends on config somehow.
-	qDebug() << "cmetrics to report:" << "Mixxx deconstructor complete.";
-	cm_writemsg_ascii(MIXXXCMETRICS_MIXXX_DESTRUCTOR_COMPLETE, "Mixxx deconstructor complete.");
-	cm_close(10);
+    qDebug() << "cmetrics to report:" << "Mixxx deconstructor complete.";
+    cm_writemsg_ascii(MIXXXCMETRICS_MIXXX_DESTRUCTOR_COMPLETE, "Mixxx deconstructor complete.");
+    cm_close(10);
 #endif
 
     qDebug() << "delete config, " << qTime.elapsed();
     delete config;
-
-// Why is this here? The (MSVC 2008) linker even complains about it.
-//#ifdef __WINDOWS__
-//    _exit(0);
-//#endif
 }
 
 int MixxxApp::noSoundDlg(void)
 {
     QMessageBox msgBox;
-	msgBox.setIcon(QMessageBox::Warning);
+    msgBox.setIcon(QMessageBox::Warning);
     msgBox.setWindowTitle("Sound Device Busy");
     msgBox.setText( "<html>Mixxx cannot access the sound device <b>"+
                     config->getValueString(ConfigKey("[Soundcard]", "DeviceMaster"))+
@@ -458,7 +529,7 @@ int MixxxApp::noSoundDlg(void)
     while(1)
     {
         msgBox.exec();
-        
+
         if (msgBox.clickedButton() == retryButton) {
             soundmanager->queryDevices();
             return 0;
@@ -468,17 +539,17 @@ int MixxxApp::noSoundDlg(void)
         } else if (msgBox.clickedButton() == reconfigureButton) {
             msgBox.hide();
             soundmanager->queryDevices();
-            
+
             // This way of opening the dialog allows us to use it synchronously
-            prefDlg->setWindowModality(Qt::ApplicationModal);
+           prefDlg->setWindowModality(Qt::ApplicationModal);
             prefDlg->exec();
             if ( prefDlg->result() == QDialog::Accepted) {
                 soundmanager->queryDevices();
                 return 0;
             }
-            
+
             msgBox.show();
-            
+
         } else if (msgBox.clickedButton() == exitButton) {
             return 1;
         }
@@ -507,14 +578,14 @@ void MixxxApp::initActions()
     playlistsNew->setShortcut(tr("Ctrl+N"));
     playlistsNew->setShortcutContext(Qt::ApplicationShortcut);
 
+    cratesNew = new QAction(tr("Add new &crate"), this);
+    cratesNew->setShortcut(tr("Ctrl+C"));
+    cratesNew->setShortcutContext(Qt::ApplicationShortcut);
+
     playlistsImport = new QAction(tr("&Import playlist"), this);
     playlistsImport->setShortcut(tr("Ctrl+I"));
     playlistsImport->setShortcutContext(Qt::ApplicationShortcut);
 
-    batchBpmDetect = new QAction(tr("Batch BPM Detect (up to 100 songs per run)"), this);
-    connect(batchBpmDetect, SIGNAL(activated()), m_pTrack, SLOT(slotBatchBPMDetection()));
-
-
 #ifdef __IPOD__
     iPodToggle = new QAction(tr("iPod &Active"), this);
     iPodToggle->setShortcut(tr("Ctrl+A"));
@@ -526,7 +597,7 @@ void MixxxApp::initActions()
     optionsBeatMark = new QAction(tr("&Audio Beat Marks"), this);
 
     optionsFullScreen = new QAction(tr("&Full Screen"), this);
-    optionsFullScreen->setShortcut(tr("Esc"));
+    optionsFullScreen->setShortcut(tr("F11"));
     optionsFullScreen->setShortcutContext(Qt::ApplicationShortcut);
     //QShortcut * shortcut = new QShortcut(QKeySequence(tr("Esc")),  this);
     //    connect(shortcut, SIGNAL(activated()), this, SLOT(slotQuitFullScreen()));
@@ -550,10 +621,6 @@ void MixxxApp::initActions()
 #ifdef __SCRIPT__
     macroStudio = new QAction(tr("Show Studio"), this);
 #endif
-#ifdef __LADSPA__
-    ladspaShow = new QAction(tr("Show LADSPA window"), this);
-#endif
-
 
     fileLoadSongPlayer1->setStatusTip(tr("Opens a song in player 1"));
     fileLoadSongPlayer1->setWhatsThis(tr("Open\n\nOpens a song in player 1"));
@@ -570,15 +637,21 @@ void MixxxApp::initActions()
     libraryRescan->setStatusTip(tr("Rescans the song library"));
     libraryRescan->setWhatsThis(tr("Rescan library\n\nRescans the song library"));
     libraryRescan->setCheckable(false);
-    connect(libraryRescan, SIGNAL(activated()), m_pTrack, SLOT(slotScanLibrary()));
+    connect(libraryRescan, SIGNAL(activated()), this, SLOT(slotScanLibrary()));
+    connect(m_pLibraryScanner, SIGNAL(scanFinished()), this, SLOT(slotEnableRescanLibraryAction()));
 
     playlistsNew->setStatusTip(tr("Create a new playlist"));
     playlistsNew->setWhatsThis(tr("New playlist\n\nCreate a new playlist"));
-    connect(playlistsNew, SIGNAL(activated()), m_pTrack, SLOT(slotNewPlaylist()));
+    connect(playlistsNew, SIGNAL(activated()), m_pLibrary, SLOT(slotCreatePlaylist()));
+
+    cratesNew->setStatusTip(tr("Create a new crate"));
+    cratesNew->setWhatsThis(tr("New crate\n\nCreate a new crate."));
+    connect(cratesNew, SIGNAL(activated()), m_pLibrary, SLOT(slotCreateCrate()));
 
     playlistsImport->setStatusTip(tr("Import playlist"));
     playlistsImport->setWhatsThis(tr("Import playlist"));
-    connect(playlistsImport, SIGNAL(activated()), m_pTrack, SLOT(slotImportPlaylist()));
+    //connect(playlistsImport, SIGNAL(activated()), m_pTrack, SLOT(slotImportPlaylist()));
+    //FIXME: Disabled due to library rework
 
     optionsBeatMark->setCheckable(false);
     optionsBeatMark->setChecked(false);
@@ -626,11 +699,6 @@ void MixxxApp::initActions()
     macroStudio->setWhatsThis(tr("Show Studio\n\nMakes the macro studio visible"));
      connect(macroStudio, SIGNAL(activated()), scriptEng->getStudio(), SLOT(showStudio()));
 #endif
-#ifdef __LADSPA__
-    ladspaShow->setStatusTip(tr("Shows the LADSPA window"));
-    ladspaShow->setWhatsThis(tr("Show LADSPA window\n\nMakes the LADSPA window visible"));
-    connect(ladspaShow, SIGNAL(activated()), this, SLOT(slotLadspa()));
-#endif
 }
 
 void MixxxApp::initMenuBar()
@@ -643,9 +711,6 @@ void MixxxApp::initMenuBar()
     helpMenu=new QMenu("&Help");
 #ifdef __SCRIPT__
     macroMenu=new QMenu("&Macro");
-#endif
-#ifdef __LADSPA__
-//    ladspaMenu=new QMenu("LADSPA");
 #endif
 
     // menuBar entry fileMenu
@@ -669,8 +734,8 @@ void MixxxApp::initMenuBar()
     libraryMenu->addAction(libraryRescan);
     libraryMenu->addSeparator();
     libraryMenu->addAction(playlistsNew);
-    libraryMenu->addAction(playlistsImport);
-    //libraryMenu->addAction(batchBpmDetect);
+    libraryMenu->addAction(cratesNew);
+    //libraryMenu->addAction(playlistsImport);
 
 #ifdef __IPOD__
     libraryMenu->addSeparator();
@@ -690,9 +755,6 @@ void MixxxApp::initMenuBar()
 #ifdef __SCRIPT__
     macroMenu->addAction(macroStudio);
 #endif
-#ifdef __LADSPA__
-    // ladspaMenu->addAction(ladspaShow);
-#endif
 
     menuBar()->addMenu(fileMenu);
     menuBar()->addMenu(libraryMenu);
@@ -701,9 +763,6 @@ void MixxxApp::initMenuBar()
     //    menuBar()->addMenu(viewMenu);
 #ifdef __SCRIPT__
     menuBar()->addMenu(macroMenu);
-#endif
-#ifdef __LADSPA__
-//    menuBar()->addMenu(ladspaMenu);
 #endif
     menuBar()->addSeparator();
     menuBar()->addMenu(helpMenu);
@@ -742,7 +801,6 @@ void MixxxApp::slotiPodToggle(bool toggle) {
 
 //     DON'T USE QFileInfo, it does a disk i/o stat on every file introducing a VERY long delay in loading from the iPod
 //       QFileInfo file(iPodMountPoint + QString(song->ipod_path).replace(':','/'));
-//       TrackInfoObject* pTrack = new TrackInfoObject(file.absolutePath(), file.fileName(), m_pBpmDetector );
 
        QString fullFilePath = iPodMountPoint + QString(song->ipod_path).mid(1).replace(':','/');
        QString filePath = fullFilePath.left(fullFilePath.lastIndexOf('/'));
@@ -776,19 +834,22 @@ void MixxxApp::slotiPodToggle(bool toggle) {
     }
     itdb_free (itdb);
 
-    qDebug() << "iPod playlist has" << m_pTrack->m_qIPodPlaylist.getSongNum() << "of"<< count <<"songs on the iPod.";
+    //qDebug() << "iPod playlist has" << m_pTrack->m_qIPodPlaylist.getSongNum() << "of"<< count <<"songs on the iPod.";
 
     view->m_pComboBox->setCurrentIndex( view->m_pComboBox->findData(TABLE_MODE_IPOD) );
-    m_pTrack->slotActivatePlaylist( view->m_pComboBox->findData(TABLE_MODE_IPOD) );
-    m_pTrack->resizeColumnsForLibraryMode();
+    //m_pTrack->slotActivatePlaylist( view->m_pComboBox->findData(TABLE_MODE_IPOD) );
+    //m_pTrack->resizeColumnsForLibraryMode();
+
+    //FIXME: Commented out above due to library rework.
 
   } else if (view->m_pComboBox->findData(TABLE_MODE_IPOD) != -1 ) {
     view->m_pComboBox->setCurrentIndex( view->m_pComboBox->findData(TABLE_MODE_LIBRARY) );
-    m_pTrack->slotActivatePlaylist( view->m_pComboBox->findData(TABLE_MODE_LIBRARY) );
+    //m_pTrack->slotActivatePlaylist( view->m_pComboBox->findData(TABLE_MODE_LIBRARY) );
+    //FIXME: library reworking
 
     view->m_pComboBox->removeItem( view->m_pComboBox->findData(TABLE_MODE_IPOD) );
     // Empty iPod model m_qIPodPlaylist
-    m_pTrack->m_qIPodPlaylist.clear();
+    //m_pTrack->m_qIPodPlaylist.clear();
 
   }
 #endif
@@ -838,11 +899,12 @@ void MixxxApp::slotFileLoadSongPlayer1()
             return;
     }
 
-    QString s = QFileDialog::getOpenFileName(this, tr("Load Song into Player 1"), config->getValueString(ConfigKey("[Playlist]","Directory")), QString("Audio (%1)").arg(MIXXX_SUPPORTED_AUDIO_FILETYPES));
+    QString s = QFileDialog::getOpenFileName(this, tr("Load Song into Player 1"), config->getValueString(ConfigKey("[Playlist]","Directory")), QString("Audio (%1)").arg(SoundSourceProxy::supportedFileExtensionsString()));
     if (!(s == QString::null)) {
-        TrackInfoObject * pTrack = m_pTrack->getTrackCollection()->getTrack(s);
-        if (pTrack)
-            m_pTrack->slotLoadPlayer1(pTrack);
+        // TODO(XXX) Lookup track in the Library and load that.
+        TrackInfoObject * pTrack = new TrackInfoObject(s);
+        TrackPointer track = TrackPointer(pTrack, &QObject::deleteLater);
+        m_pPlayer1->slotLoadTrack(track);
     }
 }
 
@@ -862,11 +924,12 @@ void MixxxApp::slotFileLoadSongPlayer2()
             return;
     }
 
-    QString s = QFileDialog::getOpenFileName(this, tr("Load Song into Player 2"), config->getValueString(ConfigKey("[Playlist]","Directory")), QString("Audio (%1)").arg(MIXXX_SUPPORTED_AUDIO_FILETYPES));
+    QString s = QFileDialog::getOpenFileName(this, tr("Load Song into Player 2"), config->getValueString(ConfigKey("[Playlist]","Directory")), QString("Audio (%1)").arg(SoundSourceProxy::supportedFileExtensionsString()));
     if (!(s == QString::null)) {
-        TrackInfoObject * pTrack = m_pTrack->getTrackCollection()->getTrack(s);
-        if (pTrack)
-            m_pTrack->slotLoadPlayer2(pTrack);
+        // TODO(XXX) Lookup track in the Library and load that.
+        TrackInfoObject * pTrack = new TrackInfoObject(s);
+        TrackPointer track = TrackPointer(pTrack, &QObject::deleteLater);
+        m_pPlayer2->slotLoadTrack(track);
     }
 }
 
@@ -882,11 +945,13 @@ void MixxxApp::slotOptionsBeatMark(bool)
 
 void MixxxApp::slotOptionsFullScreen(bool toggle)
 {
+    if (optionsFullScreen)
+        optionsFullScreen->setChecked(toggle);
 
-// Making a fullscreen window on linux and windows is harder than you could possibly imagine...
+    // Making a fullscreen window on linux and windows is harder than you could possibly imagine...
     if (toggle)
     {
-#ifdef __LINUX__
+#if defined(__LINUX__) || defined(__APPLE__)
          winpos = pos();
          // Can't set max to -1,-1 or 0,0 for unbounded?
          setMaximumSize(32767,32767);
@@ -895,7 +960,7 @@ void MixxxApp::slotOptionsFullScreen(bool toggle)
         showFullScreen();
         //menuBar()->hide();
         // FWI: Begin of fullscreen patch
-#ifdef __LINUX__
+#if defined(__LINUX__) || defined(__APPLE__)
         // Crazy X window managers break this so I'm told by Qt docs
         //         int deskw = app->desktop()->width();
         //         int deskh = app->desktop()->height();
@@ -1030,20 +1095,24 @@ void MixxxApp::slotHelpAbout()
 {
 
     DlgAbout *about = new DlgAbout(this);
+#if defined(AMD64) || defined(EM64T) || defined(x86_64)
+    about->version_label->setText(VERSION " x64");
+#elif defined(IA64)
+    about->version_label->setText(VERSION " IA64");
+#else
     about->version_label->setText(VERSION);
+#endif
     QString credits =
     QString("<p align=\"center\"><b>Mixxx %1 Development Team</b></p>"
 "<p align=\"center\">"
 "Adam Davison<br>"
 "Albert Santoni<br>"
-"Garth Dahlstrom<br>"
 "RJ Ryan<br>"
+"Garth Dahlstrom<br>"
 "Sean Pappalardo<br>"
-"Nick Guenther<br>"
 "Phillip Whelan<br>"
-"Zach Elko<br>"
-"Tom Care<br>"
-"Pawel Bartkiewicz<br>"
+"Tobias Rafreider<br>"
+"S. Brandt<br>"
 
 "</p>"
 "<p align=\"center\"><b>With contributions from:</b></p>"
@@ -1051,28 +1120,38 @@ void MixxxApp::slotHelpAbout()
 "Mark Hills<br>"
 "Andre Roth<br>"
 "Robin Sheat<br>"
-"Michael Pujos<br>"
 "Mark Glines<br>"
-"Claudio Bantaloukas<br>"
-"Pavol Rusnak<br>"
 "Mathieu Rene<br>"
 "Miko Kiiski<br>"
-"Navaho Gunleg<br>"
-"Gavin Pryke<br>"
 "Brian Jackson<br>"
 "Owen Williams<br>"
-"James Evans<br>"
-"Martin Sakmar<br>"
 "Andreas Pflug<br>"
 "Bas van Schaik<br>"
+"J&aacute;n Jockusch<br>"
+"Oliver St&ouml;neberg<br>"
+"Jan Jockusch<br>"
+"C. Stewart<br>"
+"Bill Egert<br>"
+"Zach Shutters<br>"
+"Owen Bullock<br>"
+"Bill Good<br>"
+"Graeme Mathieson<br>"
+"Sebastian Actist<br>"
+"Jussi Sainio<br>"
+"David Gnedt<br>"
+"Antonio Passamani<br>"
+"Guy Martin<br>"
+"Anders Gunnarson<br>"
 
 "</p>"
 "<p align=\"center\"><b>And special thanks to:</b></p>"
 "<p align=\"center\">"
 "Stanton<br>"
 "Hercules<br>"
+"EKS<br>"
 "Echo Digital Audio<br>"
 "Adam Bellinson<br>"
+"Alexandre Bancel<br>"
 "Melanie Thielker<br>"
 "Julien Rosener<br>"
 "Pau Arum&iacute;<br>"
@@ -1093,6 +1172,10 @@ void MixxxApp::slotHelpAbout()
 "Ben Wheeler<br>"
 "Wesley Stessens<br>"
 "Nathan Prado<br>"
+"Zach Elko<br>"
+"Tom Care<br>"
+"Pawel Bartkiewicz<br>"
+"Nick Guenther<br>"
 "</p>"
 
 "<p align=\"center\"><b>Past Contributors</b></p>"
@@ -1106,7 +1189,6 @@ void MixxxApp::slotHelpAbout()
 "Jeremie Zimmermann<br>"
 "Gianluca Romanin<br>"
 "Tim Jackson<br>"
-"J&aacute;n Jockusch<br>"
 "Stefan Langhammer<br>"
 "Frank Willascheck<br>"
 "Jeff Nelson<br>"
@@ -1124,7 +1206,22 @@ void MixxxApp::slotHelpAbout()
 "Karlis Kalnins<br>"
 "Amias Channer<br>"
 "Sacha Berger<br>"
-"</p>").arg(VERSION);
+"James Evans<br>"
+"Martin Sakmar<br>"
+"Navaho Gunleg<br>"
+"Gavin Pryke<br>"
+"Michael Pujos<br>"
+"Claudio Bantaloukas<br>"
+"Pavol Rusnak<br>"
+
+#if defined(AMD64) || defined(EM64T) || defined(x86_64)
+    "</p>").arg(VERSION " x64");
+#elif defined(IA64)
+    "</p>").arg(VERSION " IA64");
+#else
+    "</p>").arg(VERSION);
+#endif
+
 
 
     about->textBrowser->setHtml(credits);
@@ -1146,6 +1243,12 @@ void MixxxApp::rebootMixxxView() {
     int oldw = view->width();
     qDebug() << "Now in Rebootmixxview...";
 
+    // Workaround for changing skins while fullscreen, just go out of fullscreen
+    // mode. If you change skins while in fullscreen (on Linux, at least) the
+    // window returns to 0,0 but and the backdrop disappears so it looks as if
+    // it is not fullscreen, but acts as if it is.
+    slotOptionsFullScreen(false);
+
     QString qSkinPath = getSkinPath();
 
     view->rebootGUI(frame, config, qSkinPath);
@@ -1155,6 +1258,19 @@ void MixxxApp::rebootMixxxView() {
     if (oldw != view->width() || oldh != view->height() + menuBar()->height()) {
       setFixedSize(view->width(), view->height() + menuBar()->height());
     }
+
+    // these signals/slots need reconnected to the new m_pVisuals after reboot or the
+    // signals go to the wrong slots
+    if (view->m_pVisualCh1) {
+        disconnect(SIGNAL(trackDropped(QString)), this, SLOT(slotLoadPlayer1(QString)));
+        connect(view->m_pVisualCh1, SIGNAL(trackDropped(QString)),
+            this, SLOT(slotLoadPlayer1(QString)));
+    }
+    if (view->m_pVisualCh2) {
+        disconnect(SIGNAL(trackDropped(QString)), this, SLOT(slotLoadPlayer2(QString)));
+        connect(view->m_pVisualCh2, SIGNAL(trackDropped(QString)),
+            this, SLOT(slotLoadPlayer2(QString)));
+    }
 }
 
 QString MixxxApp::getSkinPath() {
@@ -1201,9 +1317,58 @@ bool MixxxApp::eventFilter(QObject *obj, QEvent *event)
 
 }
 
-void MixxxApp::slotLadspa()
+void MixxxApp::slotLoadTrackToPlayer(TrackPointer pTrack, int player) {
+    // TODO(XXX) In the future, when we support multiple decks, this method will
+    // be less of a hack.
+    if (player == 1) {
+        m_pPlayer1->slotLoadTrack(pTrack);
+    } else if (player == 2) {
+        m_pPlayer2->slotLoadTrack(pTrack);
+    }
+}
+void MixxxApp::slotLoadTrackIntoNextAvailablePlayer(TrackPointer pTrack)
 {
-#ifdef __LADSPA__
-    ladspaDlg->setHidden(false);
-#endif
+    if (ControlObject::getControl(ConfigKey("[Channel1]","play"))->get()!=1.)
+        m_pPlayer1->slotLoadTrack(pTrack, false);
+    else if (ControlObject::getControl(ConfigKey("[Channel2]","play"))->get()!=1.)
+        m_pPlayer2->slotLoadTrack(pTrack, false);
+}
+
+void MixxxApp::slotLoadPlayer1(QString location)
+{
+    // Try to get TrackInfoObject* from library, identified by location.
+    TrackDAO& trackDao = m_pLibrary->getTrackCollection()->getTrackDAO();
+    TrackPointer pTrack = trackDao.getTrack(trackDao.getTrackId(location));
+    // If not, create a new TrackInfoObject*
+    if (pTrack == NULL)
+    {
+        pTrack = TrackPointer(new TrackInfoObject(location), &QObject::deleteLater);
+    }
+    //Load the track into the Player.
+    m_pPlayer1->slotLoadTrack(pTrack);
+}
+
+void MixxxApp::slotLoadPlayer2(QString location)
+{
+    // Try to get TrackInfoObject* from library, identified by location.
+    TrackDAO& trackDao = m_pLibrary->getTrackCollection()->getTrackDAO();
+    TrackPointer pTrack = trackDao.getTrack(trackDao.getTrackId(location));
+    // If not, create a new TrackInfoObject*
+    if (pTrack == NULL)
+    {
+        pTrack = TrackPointer(new TrackInfoObject(location), &QObject::deleteLater);
+    }
+    //Load the track into the Player.
+    m_pPlayer2->slotLoadTrack(pTrack);
+}
+
+void MixxxApp::slotScanLibrary()
+{
+    libraryRescan->setEnabled(false);
+    m_pLibraryScanner->scan(config->getValueString(ConfigKey("[Playlist]","Directory")));
+}
+
+void MixxxApp::slotEnableRescanLibraryAction()
+{
+    libraryRescan->setEnabled(true);
 }
