@@ -359,8 +359,8 @@ void SoundManager::setDefaults(bool api, bool devices, bool other)
     if (other)
     {
         //Default samplerate, latency
-        m_pConfig->set(ConfigKey("[Soundcard]","Samplerate"), ConfigValue(44100));
-        m_pConfig->set(ConfigKey("[Soundcard]","Latency"), ConfigValue(64));
+        m_pConfig->set(ConfigKey("[Soundcard]","Samplerate"), ConfigValue(48000));
+        m_pConfig->set(ConfigKey("[Soundcard]","Latency"), ConfigValue(21));
     }
 }
 
@@ -468,7 +468,8 @@ int SoundManager::setupDevices()
     qDebug() << "iNumDevicesOpenedForOutput:" << iNumDevicesOpenedForOutput;
     qDebug() << "iNumDevicesOpenedForInput:" << iNumDevicesOpenedForInput;
 
-    return (iNumDevicesOpenedForOutput == 0); //Returns non-zero if we have no output devices
+    //Returns non-zero if we have no output devices
+    return (iNumDevicesOpenedForOutput == 0);
 }
 
 void SoundManager::sync()
